#define vmr	vmr_		/* parameter for Name Translation is l_ */
#define ROUTINE int
/*===========================================================================
  Copyright (C) 1995-2005 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++++++++++++  STI.FC +++++++++++++++++++++++++++++++++++++++
.LANGUAGE C
.IDENTIFICATION Module STI.FC
.COMMENTS
Module contains layer between the Image realted FORTRAN STxxxx interfaces
and the SC_interfaces written in (hopefully independent) C
.AUTHOR         K. Banse  	ESO - Garching
.KEYWORDS       standard interfaces.
.ENVIRONMENT    FORTRAN and C standards
.VERSION  [1.00] 871207:  created from SXFTOC.C
.VERSION  [2.60] 880411:  modified new version - the last one
.VERSION  [2.70] 880919:  use STRING_FCOPY to get character stuff back
.VERSION  [2.80] 881028:  START + STEP are now double
.VERSION  [2.90] 900316:  fix problems with FNAME_LEN+2
.VERSION  [3.00] 901213:  Master file. CG.
.VERSION  [3.01] 911217:  SCIGET needs to fill temporary strings. CG.

 050722		last modif

-----------------------------------------------------------------------------*/

#include <ftoc.h>
#include <ftoc_comm.h>
#include <midas_def.h>
	
static char savname[200];
static int  savdattype, saviomode, savfiltype, savmaxdim;

static int  *pnaxis, *pnpix, *pno;
static double *pstart, *pstep;

static flong2c *ppntr;

#if 0     /* ==== Original Code ==== */
SUBROUTINE STI1(name,dattype,iomode,filtype,maxdim)
CHARACTER   name;	/* IN : data frame name */
fint2c	*dattype;	/* IN : data type */
fint2c	*iomode;	/* IN : mode for opening of frame */
fint2c	*filtype;	/* IN : type no: 1 = image, 2 = mask, 3 = table  */
fint2c	*maxdim;	/* IN : maximum number of dimensions */
#else     /* ==== Generated Code === */
#define STI1	sti1_
STI1(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */
	
strncpy(savname,STRIPPED_STRING(1,5,0),199);
savname[199] = '\0';

savdattype = *PARAM(2,5,1,fint2c *);
saviomode = *PARAM(3,5,1,fint2c *);
savfiltype = *PARAM(4,5,1,fint2c *);
savmaxdim = *PARAM(5,5,1,fint2c *);
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE STI2(naxis,npix,start,step,pntr,no)
fint2c	*naxis;		/* OUT: no of dimensions of the image */
fint2c	*npix; 		/* OUT: size of each dimension  */
double	*start;	 	/* OUT: start coordinates: start is the address of an */
			/*      array of naxis elements */
double	*step; 		/* OUT: stepsizes : step is the address of an array of*/
			/*      naxis elements */
flong2c *pntr;
fint2c	*no;		/* OUT: file no. of data frame */
#else     /* ==== Generated Code === */
#define STI2	sti2_
STI2(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  va_start(Cargs);       /* <forif> */


pnaxis =PARAM(1,6,0,fint2c *);
pnpix =PARAM(2,6,0,fint2c *);
pstart =PARAM(3,6,0,double *);
pstep =PARAM(4,6,0,double *);

pno =PARAM(6,6,0,fint2c *);
ppntr =PARAM(5,6,0,flong2c *);
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE STI3(ident,cunit,status)
CHARACTER   ident;	/* OUT: ascii identifier of image */
CHARACTER   cunit; 	/* OUT: unit of each axis (16 char per unit) */
fint2c	*status;
#else     /* ==== Generated Code === */
#define STI3	sti3_
STI3(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */

char *myident, *mycunit;
char *mypntr;

int n, mynaxis, mynpix[6];
double mystart[6], mystep[6];



/* Temporary strings need to contain original fortran strings */  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */


myident=C_STRING(1,3,0);
mycunit=C_STRING(2,3,1);

*PARAM(3,3,2,fint2c *) = SCIGET(savname,savdattype,saviomode,savfiltype,savmaxdim,
                 &mynaxis,mynpix,mystart,mystep,myident,mycunit,&mypntr,pno);

*pnaxis = mynaxis;
for (n=0; n<savmaxdim; n++)
   {
   *(pnpix+n) = mynpix[n]; 
   *(pstart+n) = mystart[n]; 
   *(pstep+n) = mystep[n]; 
   }

/* 
printf("maxdim = %d, dattype = %d, pixels = %d, %d\n",savmaxdim,savdattype,*pnpix,*(pnpix+1));
printf("dattype = %d, imno = %d\n",savdattype,*pno);
*/

STRFCOPY(1,3,0,myident);
STRFCOPY(2,3,1,mycunit);


*ppntr = COMMON_INDEX(mypntr);
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE STI4(ident,cunit,status)
CHARACTER   ident;      /* IN: ascii identifier of image */
CHARACTER   cunit;      /* IN: unit of each axis (16 char per unit) */
fint2c  *status;
#else     /* ==== Generated Code === */
#define STI4	sti4_
STI4(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */

char *mypntr;  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */



*PARAM(3,3,2,fint2c *) = SCIPUT(savname,savdattype,saviomode,savfiltype,
                *pnaxis,pnpix,pstart,pstep,
                STRIPPED_STRING(1,3,0),STRIPPED_STRING(2,3,1),&mypntr,pno);

*ppntr = COMMON_INDEX(mypntr);
  ftoc_free(FORmark);    /* <forif> */
}

