#define vmr	vmr_		/* parameter for Name Translation is l_ */
#define ROUTINE int
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
  MA 02139, USA.
 
  Corresponding concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

#include <tbldef.h>
#include <ftoc.h>
#include <ftoc_comm.h>          /* VMR common */
#include <ftoc_log.h>           /* LOGICAL    */
#include <midas_def.h>

static int  savtid, savrow, savcol, savindx;

#define TBA1	tba1_
ROUTINE TBA1(tid,row,col,indx)
fint2c       *tid;
fint2c       *row;
fint2c       *col;
fint2c       *indx;

{
savtid = *tid;
savrow = *row;
savcol = *col;
savindx = *indx;
}

#define TBAMAP	tbamap_
ROUTINE TBAMAP(tid,row,col,indx,status)
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE Return the address of the COMPLETE element array
.RETURNS status
------------------------------------------------------------------*/
fint2c       *tid;
fint2c       *row;
fint2c       *col;
flong2c      *indx;
fint2c       *status;
{
      char  *mypntr;
      *status = TCAMAP(*tid,*row,*col,&mypntr);
      *indx  = COMMON_INDEX(mypntr);
}

#define TBAUNM	tbaunm_
ROUTINE TBAUNM(tid,indx,status)
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE Unmap a part of the file that was mapped.
.RETURNS status (-1 if not mapped)
-------------------------------------------------------------*/
fint2c       *tid;
fint2c       *indx;
fint2c       *status;
{
      *status = TCAUNM(*tid,(char *)&((&vmr.addr)[*indx-1]));
}

#define TBADEL	tbadel_
ROUTINE TBADEL(tid,row,col,indx,items,status)
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE Deletes  table elements.
.METHOD  Writes a NULL value in the table.
.RETURNS status
-------------------------------------------------------------*/
fint2c        *tid;
fint2c        *row;
fint2c        *col;
fint2c        *indx;
fint2c        *items;
fint2c        *status;
{
      *status  = TCADEL(*tid,*row,*col,*indx,*items);
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBWRC(items,value,status)
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE Writes table element, character string format.
.RETURNS status
-------------------------------------------------------------*/
fint2c        *items;
CHARACTER   value;
fint2c        *status;
#else     /* ==== Generated Code === */
#define TBWRC	tbwrc_
TBWRC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(3,3,1,fint2c *) = TCAWRC(savtid,savrow,savcol,savindx,*PARAM(1,3,0,fint2c *),C_STRING(2,3,0));
  ftoc_free(FORmark);    /* <forif> */ 
}

#define TBWRD	tbwrd_
ROUTINE TBWRD(items, value,status)
fint2c        *items;
double      *value;
fint2c        *status;

{
*status = TCAWRD(savtid,savrow,savcol,savindx,*items,value); 
}

#define TBWRI	tbwri_
ROUTINE TBWRI(items, value,status)
fint2c        *items;
fint2c        *value;
fint2c        *status;

{
*status = TCAWRI(savtid,savrow,savcol,savindx,*items,value);
}

#define TBWRR	tbwrr_
ROUTINE TBWRR(items, value,status)
fint2c        *items;
float       *value;
fint2c        *status;

{
*status = TCAWRR(savtid,savrow,savcol,savindx,*items,value);
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE  TBRDC(items,value,status)
fint2c        *items;
CHARACTER   value;
fint2c        *status;
#else     /* ==== Generated Code === */
#define TBRDC	tbrdc_
TBRDC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */

char myvalue[TBL_ROWLEN+1];  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */


*PARAM(3,3,1,fint2c *) = TCARDC(savtid,savrow,savcol,savindx,*PARAM(1,3,0,fint2c *),myvalue);

STRFCOPY(2,3,0, myvalue);
  ftoc_free(FORmark);    /* <forif> */
}

#define TBRDD	tbrdd_
ROUTINE TBRDD(items,value,status)
fint2c        *items;
double      *value;
fint2c        *status;

{
*status = TCARDD(savtid,savrow,savcol,savindx,*items,value);
}

#define TBRDI	tbrdi_
ROUTINE TBRDI(items, value,status)
fint2c        *items;
fint2c        *value; 
fint2c        *status;

{
*status = TCARDI(savtid,savrow,savcol,savindx,*items,value);
}

#define TBRDR	tbrdr_
ROUTINE TBRDR(items, value,status)
fint2c        *items;
float       *value;
fint2c        *status;

{
*status = TCARDR(savtid,savrow,savcol,savindx,*items,value);
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBSCC(items,value,next,status)
fint2c        *items;
CHARACTER   value;
fint2c        *next;
fint2c        *status;
#else     /* ==== Generated Code === */
#define TBSCC	tbscc_
TBSCC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(4,4,1,fint2c *)  = TCASRC(savtid,savrow,savcol,savindx,*PARAM(1,4,0,fint2c *),C_STRING(2,4,0),PARAM(3,4,1,fint2c *));
  ftoc_free(FORmark);    /* <forif> */
}

#define TBSCD	tbscd_
ROUTINE TBSCD(items,value,next,status)
fint2c        *items;
double      *value;
fint2c        *next;
fint2c        *status;

{
*status  = TCASRD(savtid,savrow,savcol,savindx,*items,value,next);
}

#define TBSCI	tbsci_
ROUTINE TBSCI(items,value,next,status)
fint2c        *items;
fint2c        *value;
fint2c        *next;
fint2c        *status;

{
*status  = TCASRI(savtid,savrow,savcol,savindx,*items,value,next);
}

#define TBSCR	tbscr_
ROUTINE TBSCR(items,value,next,status)
fint2c        *items;
float       *value;
fint2c        *next;
fint2c        *status;

{
*status  = TCASRR(savtid,savrow,savcol,savindx,*items,value,next);
}
