#define vmr	vmr_		/* parameter for Name Translation is l_ */
#define ROUTINE int
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
  MA 02139, USA.
 
  Corresponding concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++
.IDENTIFICATION tbr.fc
.LANGUAGE       C
.AUTHOR         J.D. Ponz, F. Ochsenbein (ESO-IPG))
.KEYWORDS       Table system, FORTRAN interface
.ENVIRONMENT    
.VERSION  1.0  	1 Feb 1987     Creation
.VERSION  1.1  	1 Dec 1987     Modification of the calling seq.
.VERSION  1.2  	7 Apr 1988     Add length to the calling seq.
.VERSION  1.3   16-Oct-1990: Simplified
.COMMENTS       FORTRAN 77 to C interface layer.
---------------*/

#include <tbldef.h>
#include <ftoc.h>
#include <ftoc_comm.h>		/* VMR common */
#include <ftoc_log.h>		/* For Logical Definitions */
#include <midas_def.h>

static savtid, *pin1, *pin2, *pin3;

typedef int (*FUNCTION)();

static int dtype(t)
/*++++++++++++++++
.PURPOSE Convert the datatype for TBRGET 
.RETURNS 0 for Char / 1 for Int / 2 for R*4 / 3 for Double
-----------------*/
	char	*t;	/* IN: Character Type */
{
  switch(*t) {
    case 'J':
    case 'I':	return(1);
    case 'D':	return(3);
    case 'C':
    case 'A':	return(0);
    case 'R':
	if ((t[1] == '*') && (t[2] >= '8'))	return(3);
    default:	return(2);
  }
}


static f77log(a, n)
/*++++++++++++++++
.PURPOSE Modify the logical values if required
.RETURNS -
-----------------*/
fint2c *a;	/* IN: LOGICAL value */
int  n;		/* IN: How many logical numbers */
{
	int i; int *p;

  for (p=a, i=n; --i >= 0; p++)
	*p = (*p ? F77TRUE : F77FALSE);
}

#define TBR1	tbr1_
ROUTINE TBR1(tid,in1,in2,in3)
fint2c       *tid;      /* IN : table identifier */
fint2c       *in1;
fint2c       *in2;
fint2c       *in3;

{
savtid = *tid;
pin1 = in1;
pin2 = in2;
pin3 = in3;
}

#define TBRDEL	tbrdel_
ROUTINE TBRDEL(tid, row, status)
/*++++++++++++++++
.PURPOSE F77 to C interface to TCRDEL: Delete all elements in the row
.RETURNS -
-----------------*/
fint2c       *tid; 	/* IN : table identifier */
fint2c       *row;	/* IN : Row concerned	 */
fint2c       *status;	/* OUT: status return */
{
  *status = TCRDEL(*tid, *row);
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBR2(value, null, status)
CHARACTER   value;	/* OUT: Translated Values	*/
fint2c	   *null;	/* OUT: Array of Null Flags	*/
fint2c       *status;	/* OUT: status return 		*/
#else     /* ==== Generated Code === */
#define TBR2	tbr2_
TBR2(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
 
char mybuf[TBL_ROWLEN+1];  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */


oscfill(mybuf,TBL_ROWLEN,' ');
mybuf[TBL_ROWLEN] = '\0';

*PARAM(3,3,1,fint2c *) = TCRRDC(savtid, *pin1, *pin2, pin3, mybuf,PARAM(2,3,1,fint2c *));
STRFCOPY(1,3,0, mybuf);
f77log (PARAM(2,3,1,fint2c *), *pin2);		/* Modify the NULL flag */
  ftoc_free(FORmark);    /* <forif> */
}

#define TBR3	tbr3_
ROUTINE TBR3( value, null, status)
double     *value;	/* OUT: Translated Values	*/
fint2c	   *null;	/* OUT: Array of Null Flags	*/
fint2c       *status;	/* OUT: status return 		*/

{ 
*status = TCRRDD(savtid, *pin1, *pin2, pin3, value, null);
f77log (null, *pin2);           /* Modify the NULL flag */
}

#define TBR4	tbr4_
ROUTINE TBR4(value, null, status)
fint2c       *value;	/* OUT: Translated Values	*/
fint2c	   *null;	/* OUT: Array of Null Flags	*/
fint2c       *status;	/* OUT: status return 		*/

{ 
*status = TCRRDI(savtid, *pin1, *pin2, pin3, value, null);
f77log (null, *pin2);		/* Modify the NULL flag */
}

#define TBR5	tbr5_
ROUTINE TBR5(value, null, status)
float      *value;	/* OUT: Translated Values	*/
fint2c	   *null;	/* OUT: Array of Null Flags	*/
fint2c       *status;	/* OUT: status return 		*/

{
*status = TCRRDR(savtid, *pin1, *pin2, pin3, value, null);
f77log (null, *pin2);           /* Modify the NULL flag */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBR6(type,value, null, status)
CHARACTER   type;	/* IN : DataType as R D I	*/
fint2c       *value;	/* OUT: Translated Values	*/
fint2c	   *null;	/* OUT: 1 if Element Null	*/
fint2c       *status;	/* OUT: status return 		*/
#else     /* ==== Generated Code === */
#define TBR6	tbr6_
TBR6(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
 
FUNCTION routine;
static	FUNCTION choice[] = {(FUNCTION) TBR2,(FUNCTION) TBR4,(FUNCTION) TBR5,(FUNCTION) TBR3 };  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */


routine  = choice[dtype(C_STRING(1,4,0))];
(*routine)(PARAM(2,4,1,fint2c *),PARAM(3,4,1,fint2c *),PARAM(4,4,1,fint2c *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBR7(text,found,status)
CHARACTER   text;	/* IN : Text of columns, e.g. @1..20,50 */
fint2c       *found;	/* OUT: How many ranges were found	*/
fint2c       *status;	/* OUT: status return */
#else     /* ==== Generated Code === */
#define TBR7	tbr7_
TBR7(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(3,3,1,fint2c *) = TCRSEL(savtid, STRIPPED_STRING(1,3,0), *pin1,
	pin2, pin3,PARAM(2,3,1,fint2c *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE TBR8(value, status)
CHARACTER   value;	/* IN : Input Values		*/
fint2c       *status;	/* OUT: status return 		*/
#else     /* ==== Generated Code === */
#define TBR8	tbr8_
TBR8(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */
 
*PARAM(2,2,1,fint2c *) = TCRWRC(savtid, *pin1, *pin2, pin3, C_STRING(1,2,0));
  ftoc_free(FORmark);    /* <forif> */
}

#define TBRWRD	tbrwrd_
ROUTINE TBRWRD(tid, row, nc, cols, value, status)
/*++++++++++++++++
.PURPOSE F77 interface to TCRWRD (Write in Double)
.RETURNS -
-----------------*/
fint2c       *tid; 	/* IN : table identifier 	*/
fint2c       *row;	/* IN : Row concerned	 	*/
fint2c       *nc;		/* IN : Size of cols array 	*/
fint2c       *cols;	/* IN : Column Selection	*/
double     *value;	/* IN : Input Values		*/
fint2c       *status;	/* OUT: status return 		*/
{ 
  *status = TCRWRD(*tid, *row, *nc, cols, value);
}

#define TBRWRI	tbrwri_
ROUTINE TBRWRI(tid, row, nc, cols, value, status)
/*++++++++++++++++
.PURPOSE F77 interface to TCRWRI (Write in Integer*4)
.RETURNS -
-----------------*/
fint2c       *tid; 	/* IN : table identifier 	*/
fint2c       *row;	/* IN : Row concerned	 	*/
fint2c       *nc;		/* IN : Size of cols array 	*/
fint2c       *cols;	/* IN : Column Selection	*/
fint2c       *value;	/* IN : Input Values		*/
fint2c       *status;	/* OUT: status return 		*/
{ 
  *status = TCRWRI(*tid, *row, *nc, cols, value);
}

#define TBRWRR	tbrwrr_
ROUTINE TBRWRR(tid, row, nc, cols, value, status)
/*++++++++++++++++
.PURPOSE F77 interface to TCRWRR (Write in Float)
.RETURNS -
-----------------*/
fint2c       *tid; 	/* IN : table identifier 	*/
fint2c       *row;	/* IN : Row concerned	 	*/
fint2c       *nc;		/* IN : Size of cols array 	*/
fint2c       *cols;	/* IN : Column Selection	*/
float      *value;	/* IN : Input Values		*/
fint2c       *status;	/* OUT: status return 		*/
{ 
  *status = TCRWRR(*tid, *row, *nc, cols, value);
}

